% SonnetCallEm will call Sonnet EM
% to simulate the Sonnet Project File.
%
% This function has the following parameters:
%   1) The filename of the project to be simulated
%   2) (Optional) a string that defines extra options
%       in the format explained below.
%
% Options are passed as a single string. Order of option switches 
% doesnt matter and unknown option switches are ignored.
%
% Supported option switches:
%   '-c'              To clean the project data first
%   '-x'              To not clean the project data first (default)
%   '-w'              To display a simulation status window (default)
%   '-t'              To not display a simulation status window
%   '-r'              To run the simulation instantaneously (default)
%   '-p'              To not run the simulation instantaneously (requires status window)
%   '-v' <VERSION>    To use a particular version of Sonnet to do the simulation
%   '-s' <DIRECTORY>  To manually specify the Sonnet directory to
%                      use for the simulation. The directory may either
%                      be the base Sonnet directory or the version's bin
%                      directory.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SonnetLab, all included documentation, all included examples
% and all other files (unless otherwise specified) are copyrighted by Sonnet Software
% in 2011 with all rights reserved.
%
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS". ANY AND
% ALL EXPRESS OR IMPLIED WARRANTIES ARE DISCLAIMED. UNDER NO CIRCUMSTANCES AND UNDER
% NO LEGAL THEORY, TORT, CONTRACT, OR OTHERWISE, SHALL THE COPYWRITE HOLDERS,  CONTRIBUTORS,
% MATLAB, OR SONNET SOFTWARE BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR
% CONSEQUENTIAL DAMAGES OF ANY CHARACTER INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOSS OF
% GOODWILL, WORK STOPPAGE, COMPUTER FAILURE OR MALFUNCTION, OR ANY AND ALL OTHER COMMERCIAL
% DAMAGES OR LOSSES, OR FOR ANY DAMAGES EVEN IF THE COPYWRITE HOLDERS, CONTRIBUTORS, MATLAB,
% OR SONNET SOFTWARE HAVE BEEN INFORMED OF THE POSSIBILITY OF SUCH DAMAGES, OR FOR ANY CLAIM
% BY ANY OTHER PARTY.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [aStatus aMessage]=SonnetCallEm(theFilename,theProjectFilePath,theOptions)

if nargin == 2
    theOptions='';
end

% If they specified a specific version of Sonnet to use then use that version
if ~isempty(strfind(theOptions,'-v')) || ~isempty(strfind(theOptions,'-V'))
    % Parse the input string
    [aParcialString, aRemainingString] = strtok(theOptions,'vV'); %#ok<ASGLU>
    
    % Read in the version number
    aRemainingString=aRemainingString(2:length(aRemainingString));
    aSonnetVersion=sscanf(aRemainingString,'%s',1);
    
    Path=SonnetPath(aSonnetVersion);
    
elseif ~isempty(strfind(theOptions,'-s')) || ~isempty(strfind(theOptions,'-S'))
    % Parse the input string
    [aParcialString, aRemainingString] = strtok(theOptions,'sS'); %#ok<ASGLU>
    
    % Read in the version number
    aRemainingString=aRemainingString(2:length(aRemainingString));
    
    % Store the path
    Path='';
    for iCounter=1:length(aRemainingString)
        if aRemainingString(iCounter) ~= '-'
            Path=[Path aRemainingString(iCounter)]; %#ok<AGROW>
        else
           break; 
        end
    end
    
    % Remove any quotation marks
    Path=strrep(Path,'"','');
    Path=strtrim(Path);
    Path=strrep(Path,'\bin','');
    
    % If the specified location does not have \bin\em.exe
    % then it is not a valid Sonnet directory.
    if ~exist([Path '\bin\em.exe'],'file')
       error(['Invalid Sonnet Directory Specified: ' Path]); 
    end
    
else
    Path=SonnetPath();
end

% Call Sonnet to unlock the project
if isempty(strfind(theOptions,'-l')) && isempty(strfind(theOptions,'-L'))
    aCallToSystem=['"' Path '\bin\soncmd.exe" -unlock "' theFilename '"'];
    [aStatus aMessage]=system(aCallToSystem);
    
    if aStatus
        error(['Failed to call soncmd to unlock the project: ' aMessage]);
    end
end

% If the 'C' flag was given delete the folder for the simulation to clean the data
if ~isempty(strfind(theOptions,'-c')) || ~isempty(strfind(theOptions,'-C')) % If they didnt pass an x value
    aProject=SonnetProject(theFilename);
    aProject.cleanProject;
end

% If the path doesn't have a trailing '\' then append one
if ~isempty(theProjectFilePath) && theProjectFilePath(length(theProjectFilePath)) ~= '\'
    theProjectFilePath = [theProjectFilePath '\'];
end

% If the folder for simulation results doesn't exist create it
aFolderLocation=[theProjectFilePath '\sondata\' strrep(theFilename,'.son','')];
[~,result]=mkdir(aFolderLocation); %#ok<NASGU>

aStatus=[];
aMessage='';

% Prep System Call to invoke EM on the .son file
if isempty(strfind(theOptions,'-t')) && isempty(strfind(theOptions,'-T'))
    % If we are drawing an EM Status window
    if ~isempty(strfind(theOptions,'-p')) || ~isempty(strfind(theOptions,'-P'))
        % If we are going to start out paused
        aCallToSystem=['"' Path '\bin\emstatus.exe" "' theProjectFilePath theFilename '" -LogFile "' aFolderLocation filesep 'SonnetLabLog.log" &'];
    else
        % If we are running right away
        aCallToSystem=['"' Path '\bin\emstatus.exe" -Run "' theProjectFilePath theFilename '" -LogFile "' aFolderLocation filesep 'SonnetLabLog.log" &'];
    end
else
    % If we are going to do the simulation without a GUI
    aCallToSystem=['"' Path '\bin\em.exe" "' theFilename '"'];
end

% Make the call and wait for the simulation to complete successfully or to fail
if ~isempty(strfind(theOptions,'-t')) || ~isempty(strfind(theOptions,'-T'))
    [aStatus aMessage]=system(aCallToSystem);
else
    isSimulationNotDone=true;
    aLogFileLocation=[aFolderLocation filesep 'SonnetLabLog.log'];
    
    % if the log file exists then delete it
    if ~isempty(dir(aLogFileLocation))
        warning off all
        delete(aLogFileLocation);
        warning on all
    end
    
    % Run the simulation
    system(aCallToSystem);
       
    while true
        
        % This pause needs to be here (cut it in half
        % or something if necessary but leave something there)
        % because otherwise we have full utilization
        % for of matlab while the simulation is running.
        pause(1);
        
        % if the log file doesnt exist yet then wait for it to exist
        % if 5 seconds go by and there is no log file yet then throw
        % an error.
        aFid=fopen(aLogFileLocation);
        aAmountOfTimeWaitedSoFar=0;
        aNumberOfSends=0;
        while aFid == -1
            aFid=fopen(aLogFileLocation);
            pause(1);
            aAmountOfTimeWaitedSoFar=aAmountOfTimeWaitedSoFar+1;
            if aAmountOfTimeWaitedSoFar > 5
                system(aCallToSystem);
                if aNumberOfSends==3
                    error('Sonnet never created a log file.')
                else
                    aNumberOfSends=aNumberOfSends+1;
                end
            end
        end
        
        % Search the file to see if we are done with the simulation
        while feof(aFid)==0
            aFileLine=fgetl(aFid);
            if ~isempty(strfind(aFileLine,'Analysis stopped'))
                aStatus=1;
                aMessage='Analysis stopped';
            end
            if ~isempty(strfind(aFileLine,'EM abnormally terminated'))
                aStatus=1;
                aMessage='EM abnormally terminated';
            end
            if ~isempty(strfind(aFileLine,'Analysis completed successfully'))
                aStatus=0;
                aMessage='';
            end            
            if ~isempty(strfind(aFileLine,'Log file closed'))
                fclose(aFid);
                return;
            end
        end
        fclose(aFid);
    end
end
